CREATE OR REPLACE FUNCTION public.getfallbacktext(p_translation character varying, p_fallback character varying, p_reference text) RETURNS TABLE(translation character varying, text character varying)
    LANGUAGE SQL
    AS $$
SELECT translation, text FROM (SELECT
    0 AS ordinal,
    p_translation AS translation,
    gettext(p_translation, p_reference) AS text
UNION SELECT
    1 AS ordinal,
    p_fallback AS translation,
    gettext(p_fallback, p_reference) AS text
UNION SELECT
    3 AS ordinal,
    getdefaulttranslation() AS translation,
    getdefaulttext(p_reference) AS text
) AS subq
WHERE text IS NOT NULL
ORDER BY ordinal ASC LIMIT 1;
$$;

CREATE TABLE IF NOT EXISTS public.pg_categories(
    tableid int NOT NULL,
    name text NOT NULL,
    tablename text NOT NULL,
    PRIMARY KEY(name)
);
ALTER TABLE public.pg_categories OWNER TO pgdb;

INSERT INTO public.pg_categories(tablename, name, tableid)
VALUES
    ('c_name', 'name', 10), ('c_z_kingdom', 'z_kingdom', 15), ('c_a_kingdom', 'a_kingdom', 20),
    ('c_will', 'will', 25), ('c_bread', 'bread', 30), ('c_because', 'because', 35),
    ('c_forgive', 'forgive', 40), ('c_debtors', 'debtors', 45), ('c_tempt', 'tempt', 50),
    ('c_deliver', 'deliver', 60), ('c_glory', 'glory', 70)
ON CONFLICT DO NOTHING;

CREATE OR REPLACE FUNCTION public.getprayer(p_name text[], p_translation character varying [], p_reference text [])
RETURNS TABLE(ordinal integer, category text, translation character varying, reference text, txt text)
    LANGUAGE plpgsql
    AS $$
DECLARE
  r record;
BEGIN
FOR r IN
  SELECT tableid, c.name, p.translation AS translation, p.reference AS reference FROM pg_categories AS c JOIN UNNEST(p_name, p_translation, p_reference) AS p(name, translation, reference) ON c.name = p.name
LOOP
  SELECT t.translation, t.text INTO translation, txt FROM getfallbacktext(r.translation, getdefaulttranslation(), r.reference) AS t;
  RETURN QUERY SELECT r.tableid, r.name, translation, normalizeverse(r.reference::character varying)::text, txt;
END LOOP;
RETURN;
END
$$;


--
-- Name: getrandomsample(text, bigint); Type: FUNCTION; Schema: public; Owner: pgdb
--

CREATE OR REPLACE FUNCTION public.getrandomsample(p_name text, p_count bigint) RETURNS TABLE(ordinal integer, category text, translation character varying, reference text)
    LANGUAGE plpgsql
    AS $$
DECLARE
  v_estimate bigint;
  v_percentage double precision;
  v_table_id int;
  v_table text;
BEGIN
SELECT tableid, tablename, reltuples INTO v_table_id, v_table, v_estimate
FROM pg_categories
JOIN pg_class ON tablename = relname
WHERE p_name = name;
SELECT 1000::double precision / v_estimate INTO v_percentage;
RETURN QUERY EXECUTE 'SELECT '|| v_table_id ||' AS ordinal,
  '''|| p_name ||''' AS category, translation, reference
FROM ' || v_table || ' TABLESAMPLE bernoulli ('|| v_percentage ||')
LIMIT ' || p_count;
END
$$;


ALTER FUNCTION public.getrandomsample(p_table text, p_count bigint) OWNER TO pgdb;

--
-- Name: pg_random_view; Type: VIEW; Schema: public; Owner: pgdb
--

-- DROP VIEW IF EXISTS public.pg_random_view CASCADE;
CREATE OR REPLACE VIEW public.pg_random_view AS
 SELECT getrandomsample.ordinal,
    getrandomsample.category,
    getrandomsample.translation,
    getrandomsample.reference,
    public.gettext(getrandomsample.translation, (getrandomsample.reference)::character varying) AS txt
   FROM public.getrandomsample('name'::text, (1)::bigint) getrandomsample(ordinal, category, translation, reference)
UNION
 SELECT getrandomsample.ordinal,
    getrandomsample.category,
    getrandomsample.translation,
    getrandomsample.reference,
    public.gettext(getrandomsample.translation, (getrandomsample.reference)::character varying) AS txt
   FROM public.getrandomsample('bread'::text, (1)::bigint) getrandomsample(ordinal, category, translation, reference)
UNION
 SELECT getrandomsample.ordinal,
    getrandomsample.category,
    getrandomsample.translation,
    getrandomsample.reference,
    public.gettext(getrandomsample.translation, (getrandomsample.reference)::character varying) AS txt
   FROM public.getrandomsample('debtors'::text, (1)::bigint) getrandomsample(ordinal, category, translation, reference)
UNION
 SELECT getrandomsample.ordinal,
    getrandomsample.category,
    getrandomsample.translation,
    getrandomsample.reference,
    public.gettext(getrandomsample.translation, (getrandomsample.reference)::character varying) AS txt
   FROM public.getrandomsample('because'::text, (1)::bigint) getrandomsample(ordinal, category, translation, reference)
UNION
 SELECT getrandomsample.ordinal,
    getrandomsample.category,
    getrandomsample.translation,
    getrandomsample.reference,
    public.gettext(getrandomsample.translation, (getrandomsample.reference)::character varying) AS txt
   FROM public.getrandomsample('tempt'::text, (1)::bigint) getrandomsample(ordinal, category, translation, reference)
UNION
 SELECT getrandomsample.ordinal,
    getrandomsample.category,
    getrandomsample.translation,
    getrandomsample.reference,
    public.gettext(getrandomsample.translation, (getrandomsample.reference)::character varying) AS txt
   FROM public.getrandomsample('deliver'::text, (1)::bigint) getrandomsample(ordinal, category, translation, reference)
UNION
 SELECT getrandomsample.ordinal,
    getrandomsample.category,
    getrandomsample.translation,
    getrandomsample.reference,
    public.gettext(getrandomsample.translation, (getrandomsample.reference)::character varying) AS txt
   FROM public.getrandomsample('glory'::text, (1)::bigint) getrandomsample(ordinal, category, translation, reference)
UNION
 SELECT getrandomsample.ordinal,
    getrandomsample.category,
    getrandomsample.translation,
    getrandomsample.reference,
    public.gettext(getrandomsample.translation, (getrandomsample.reference)::character varying) AS txt
   FROM public.getrandomsample('will'::text, (1)::bigint) getrandomsample(ordinal, category, translation, reference)
UNION
 SELECT getrandomsample.ordinal,
    getrandomsample.category,
    getrandomsample.translation,
    getrandomsample.reference,
    public.gettext(getrandomsample.translation, (getrandomsample.reference)::character varying) AS txt
   FROM public.getrandomsample('a_kingdom'::text, (1)::bigint) getrandomsample(ordinal, category, translation, reference)
UNION
 SELECT getrandomsample.ordinal,
    getrandomsample.category,
    getrandomsample.translation,
    getrandomsample.reference,
    public.gettext(getrandomsample.translation, (getrandomsample.reference)::character varying) AS txt
   FROM public.getrandomsample('z_kingdom'::text, (1)::bigint) getrandomsample(ordinal, category, translation, reference)
  ORDER BY 1;

ALTER TABLE public.pg_random_view OWNER TO pgdb;


--
-- Name: pg_random_view_default_if_null; Type: VIEW; Schema: public; Owner: pgdb
--

-- DROP VIEW IF EXISTS public.pg_random_view_default_if_null CASCADE;
CREATE OR REPLACE VIEW public.pg_random_view_default_if_null AS
 SELECT category, (rec).translation, reference, (rec).text AS txt FROM (
        SELECT subq.category,
	    subq.reference,
            public.getfallbacktext(subq.translation, getdefaulttranslation(), subq.reference) AS rec
        FROM public.pg_random_view subq) AS q;

ALTER TABLE public.pg_random_view_default_if_null OWNER TO pgdb;


--
-- Name: pg_view; Type: VIEW; Schema: public; Owner: pgdb
--

CREATE OR REPLACE VIEW public.pg_view AS
 SELECT reference.translation,
    reference.reference,
    public.gettext(reference.translation, (reference.reference)::character varying) AS txt
   FROM public.reference;

ALTER TABLE public.pg_view OWNER TO pgdb;


--
-- Name: pg_view_default_if_null; Type: VIEW; Schema: public; Owner: pgdb
--

CREATE OR REPLACE VIEW public.pg_view_default_if_null AS
 SELECT
        CASE
            WHEN (subq.txt IS NULL) THEN public.getdefaulttranslation()
            ELSE subq.translation
        END AS translation,
    subq.reference,
        CASE
            WHEN (subq.txt IS NULL) THEN public.getdefaulttext((subq.reference)::character varying)
            ELSE subq.txt
        END AS txt
   FROM ( SELECT reference.translation,
            reference.reference,
            public.gettext(reference.translation, (reference.reference)::character varying) AS txt
           FROM public.reference) subq;

ALTER TABLE public.pg_view_default_if_null OWNER TO pgdb;


